;; [[file:arroyo-emacs.org::*\[\[id:arroyo/system-cache\]\[Arroyo System Cache\]\] Emacs table][[[id:arroyo/system-cache][Arroyo System Cache]] Emacs table:1]]
(require 'arroyo-utils)

(defcustom arroyo-emacs-init-location (expand-file-name "~/arroyo-nix/files/init.el")
  "Where `arroyo-emacs-generate-init' writes the amalgamated init file."
  :group 'arroyo
  :type 'string)

(dolist (kw '("ARROYO_EMACS_MODULE" "ARROYO_EMACS_TANGLED"))
  (add-to-list 'arroyo-db-keywords kw nil #'equal))

(add-to-list 'arroyo-db--schemata
             '(emacs
               [(file     :not-null)
                (tangled  :not-null)
                (module   :not-null)]))
;; [[id:arroyo/system-cache][Arroyo System Cache]] Emacs table:1 ends here

;; [[file:arroyo-emacs.org::*\[\[id:arroyo/system-cache\]\[Arroyo System Cache\]\] Emacs table][[[id:arroyo/system-cache][Arroyo System Cache]] Emacs table:2]]
(defun arroyo-emacs-update-db (&optional keywords-too)
  (interactive "P")
  (when keywords-too
    (arroyo-db-update-all-roam-files))
  (if (org-roam-file-p)
      (arroyo-db-query [:delete :from emacs :where (= file $s1)]
                       (buffer-file-name))
    (arroyo-db-query [:delete :from emacs]))
  (when-let* ((module-args
               (if (org-roam-file-p)
                   (list [:select [file "" value] :from keywords
                          :where (= keyword $s1) :and (= file $s2)]
                         "ARROYO_EMACS_MODULE" 
                         (buffer-file-name))
                 (list [:select [file "" value] :from keywords
                        :where (= keyword $s1)]
                       "ARROYO_EMACS_MODULE")))
              (modules (apply #'arroyo-db-query module-args)))
    (arroyo-db-query [:insert :into emacs :values $v1]
                     (-map (pcase-lambda (`(,file ,_ ,module))
                                (vector file (arroyo-emacs--figure-tangled-location file) module))
                              modules))))

(defun arroyo-emacs--figure-tangled-location (file-path)
  (->> file-path
       (arroyo-db-query [:select value :from keywords
                         :where (= file $s1)
                         :and (= keyword "ARROYO_EMACS_MODULE")])
       (caar)
       (format "%s/cce/%s.el" org-roam-directory)))

(add-to-list 'arroyo-db-update-functions #'arroyo-emacs-update-db)
;; [[id:arroyo/system-cache][Arroyo System Cache]] Emacs table:2 ends here

;; [[file:arroyo-emacs.org::*Arroyo Emacs =init.el= generator][Arroyo Emacs =init.el= generator:1]]
(defun arroyo-emacs-generate-init ()
  "Write an emacs `init.el' to `arroyo-emacs-init-location' using
   tangled files spread throughout an org-roam wiki."
  (interactive)
  (arroyo-emacs-update-db)
  (let ((dest arroyo-emacs-init-location)
        (files (arroyo-emacs--ordered-files)))
    (with-current-buffer (find-file-noselect dest)
      (erase-buffer)
      (dolist (file files)
        (insert-file-contents file)
        (goto-char (point-max)))
      (save-buffer))))

(defun arroyo-emacs--module-pair-for-rel-file (unordered rel-file)
  (second (-find (lambda (elt)
                   (equal (expand-file-name rel-file org-roam-directory)
                          (car elt)))
                 unordered)))

(defun arroyo-emacs--ordered-files ()                                                     ;; (ref:ordered-files)
  (let* ((unordered (arroyo-db-query [:select [file tangled] :from emacs]))               ;; (ref:unordered)
         (unordered-filenames (-map #'car unordered))                                     ;; (ref:unordered-filenames)
         (order (arroyo-utils-order-modules unordered-filenames))                         ;; (ref:order-modules)
         (partialfn (apply-partially #'arroyo-emacs--module-pair-for-rel-file unordered))
         (ordered (-map partialfn order)))
    (->>
     ordered
     (-filter #'identity)
     (-uniq))))
;; Arroyo Emacs =init.el= generator:1 ends here

;; [[file:arroyo-emacs.org::*\[\[id:cce/literate_programming\]\[Literate Programming\]\] helpers][[[id:cce/literate_programming][Literate Programming]] helpers:1]]
(defun arroyo-emacs-epkgs ()
  (->> (arroyo-db-query [:select [file value] :from keywords
                         :where (= keyword "ARROYO_HOME_EPKGS")])
       (-map #'second)
       (-map (lambda (path)
               (let ((path (expand-file-name path "~/arroyo-nix")))
                 (with-current-buffer (find-file-noselect path)
                   (let ((bs (buffer-string)))
                     (kill-this-buffer)
                     bs)))))
       (s-join "\n")))
(add-to-list 'arroyo-db-keywords "ARROYO_HOME_EPKGS" nil #'equal)
;; [[id:cce/literate_programming][Literate Programming]] helpers:1 ends here

;; [[file:arroyo-emacs.org::generate_epkg_overrides][generate_epkg_overrides]]
(arroyo-emacs-epkgs)
;; generate_epkg_overrides ends here

;; [[file:arroyo-emacs.org::*need a reverse of =ARROYO_MODULE_WANTS= for specifying "wanted-by"s][need a reverse of =ARROYO_MODULE_WANTS= for specifying "wanted-by"s:1]]
(provide 'arroyo-emacs)
;; need a reverse of =ARROYO_MODULE_WANTS= for specifying "wanted-by"s:1 ends here
