;; [[file:arroyo-home-manager.org::+begin_src emacs-lisp][No heading:1]]
(add-to-list 'arroyo-db-keywords "ARROYO_HOME_MODULE" nil #'equal)
(add-to-list 'arroyo-db-keywords "CCE_HOME_MODULE" nil #'equal)
(add-to-list 'arroyo-db--schemata
             '(home-manager
               [(file        :not-null)
                ;(file-hash   :not-null)
                (home-module :not-null)
                role]))
;; No heading:1 ends here

;; [[file:arroyo-home-manager.org::+begin_src emacs-lisp][No heading:2]]
(defun arroyo-home-manager-update-db (&optional keywords-too)
  (interactive "P")
  (when keywords-too
    (arroyo-db-update-all-roam-files))
  (if (org-roam-file-p)
      (arroyo-db-query [:delete :from home_manager :where (= file $s1)]
                       (buffer-file-name))
    (arroyo-db-query [:delete :from home_manager]))
  (when-let* ((args
               (if (org-roam-file-p)
                   (list [:select [file value] :from keywords
                          :where (in keyword $v1) :and (= file $s2)]
                         (vector "ARROYO_HOME_MODULE" "CCE_HOME_MODULE")
                         (buffer-file-name))
                 (list [:select [file value] :from keywords
                        :where (in keyword $v1)]
                       (vector "ARROYO_HOME_MODULE" "CCE_HOME_MODULE"))))
              (results (apply #'arroyo-db-query args))
              ;; attach role
              (results (-map (pcase-lambda (`(,file ,value))
                               (list file value (arroyo-db-get "ARROYO_NIXOS_ROLE" file)))
                             results)))
    (arroyo-db-query [:insert :into home-manager :values $v1]
                     (-map (lambda (result) (apply #'vector result))
                           results))))
(add-to-list 'arroyo-db-update-functions #'arroyo-home-manager-update-db)
;; No heading:2 ends here

;; [[file:arroyo-home-manager.org::arroyo-home-manager-imports][arroyo-home-manager-imports]]
(defun arroyo-home-manager-imports1 ()
  (->> [:select [home-module file] :from home-manager]
       (arroyo-db-query)))

(defun arroyo-home-manager-imports (&optional role)
  (if-let* ((imports (arroyo-home-manager-imports1))
            (no-role? (not role)))
      (->> imports
           (-map #'car)
           (-sort #'s-less?)
           (s-join "\n"))
    (progn
      (let* ((all-files (-map #'cadr imports))
             (role-only-files (-map #'car (arroyo-db-query [:select file :from home-manager :where (like role $r1)] (format "%%%s%%" role))))
             (exclude-these-files (arroyo-db-by-keyword "ARROYO_NIXOS_EXCLUDE" role))
             ;; also need to exclude things whose ARROYO_NIXOS_ROLE is not `role'!
             (not-my-role-files
              (-map #'car
                    (arroyo-db-query [:select file :from home-manager
                                      :where (!= role $s1)]
                                     (list role))))
             ;; combine all those lists in to a single authoritative list.
             (amalgamated-files
              (-union role-only-files
                      (->> all-files
                           (--remove (-contains? exclude-these-files it))
                           (--remove (-contains? not-my-role-files it)))))
             ;; make them relative to org-roam-directory for the module orderer
             (rel-files (--map (file-relative-name it org-roam-directory) amalgamated-files)))
        ;; filter imports
        (->> rel-files
             (arroyo-utils-order-modules)
             (-map (lambda (file)
                         (caar (arroyo-db-query [:select home-module :from home-manager
                                                 :where (= file $s1)]
                                                (expand-file-name file org-roam-directory)))))
             (-filter #'identity)
             (-sort #'s-less?))))))
;; arroyo-home-manager-imports ends here

;; [[file:arroyo-home-manager.org::+begin_src emacs-lisp][No heading:4]]
(provide 'arroyo-home-manager)
;; No heading:4 ends here
