# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:1]]
import sys

from sqlmodel import Session
from . import models
from .arroyo_rs import parse_file, InvalidDocError


def persist_one_file(s: Session, path: str) -> models.Document | None:
  existing_doc = s.get(models.Document, path)
  if existing_doc and not models.document.hash_updated(existing_doc):
    return None
  if existing_doc:
    s.delete(existing_doc)
# Persisting a single file to the Database:1 ends here

# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:2]]
  try:
    doc = parse_file(path)
  except InvalidDocError:
    sys.stderr.write(f"Skipping invalid doc {path}\n")
    return None
# Persisting a single file to the Database:2 ends here

# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:3]]
  kws = map(lambda kw: kw.keyword, doc.keywords)
  kws = filter(lambda kw: kw.upper() == "ARCOLOGY_KEY", kws)
  create_documents = len(list(kws)) > 0
# Persisting a single file to the Database:3 ends here

# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:4]]
  keywords = models.Keyword.from_native_doc(doc)
  [s.merge(keyword) for keyword in keywords]
# Persisting a single file to the Database:4 ends here

# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:5]]
  if create_documents:
    tags = models.Tag.from_native_doc(doc)
    references = models.Reference.from_native_doc(doc)
    headings = models.Heading.from_native_doc(doc)
    document = models.Document.from_native_doc(doc)
    s.add(document)
  
    [s.merge(tag) for tag in tags]
    [s.merge(reference) for reference in references]
    [s.merge(heading) for heading in headings]
    s.commit()
    s.refresh(document)
  
    sys.stderr.write(f"Persisted document for {path}\n")
    return document
# Persisting a single file to the Database:5 ends here

# [[file:../arroyo-native-parser.org::*Persisting a single file to the Database][Persisting a single file to the Database:6]]
  else:
    sys.stderr.write(f"Only keywords for doc {path}\n")
    s.commit() # can't batch because of the upsert of tags...
# Persisting a single file to the Database:6 ends here
