# [[file:../arroyo-native-parser.org::*Click command wrapper][Click command wrapper:1]]
import os
import click
import glob

from . import models, persist_one_file

from sqlmodel import Session
# Click command wrapper:1 ends here

# [[file:../arroyo-native-parser.org::*Click command wrapper][Click command wrapper:2]]
@click.group()
def cli():
  pass

@cli.command()
@click.option("--source", "-s", help="Org source directory", default="~/org")
@click.option("--file-glob", "-g", help="File search glob", default="**/*.org")
@click.option("--dest", "-d", help="Sqlite Database Location", default="./arroyo.db")
def generate_db(source, dest, file_glob):
  engine = models.make_engine(dest)

  expanded_src = os.path.expanduser(source)
  files = glob.glob(file_glob, root_dir=expanded_src, recursive=True)
  files = map(lambda it: os.path.join(expanded_src, it), files)
  files = list(filter(os.path.isfile, files))

  with Session(engine) as session:
    docs = [
      x for x in [
        persist_one_file(session, path)
        for path in files
      ] if x is not None
    ]

  print(f"Parsed {len(files)} files.")
  print(f"Persisted {len(docs)} docs.")
# Click command wrapper:2 ends here

# [[file:../arroyo-native-parser.org::*Click command wrapper][Click command wrapper:3]]
if __name__ == "__main__":
  cli()
# Click command wrapper:3 ends here
