# [[file:../../arroyo-native-parser.org::*=arroyo.models.Document=][=arroyo.models.Document=:1]]
from typing import Optional, List

from sqlmodel import Field, SQLModel, Relationship
# =arroyo.models.Document=:1 ends here

# [[file:../../arroyo-native-parser.org::*=arroyo.models.Document=][=arroyo.models.Document=:2]]
from arroyo.models.keyword import Keyword
from arroyo.models.heading import Heading

import arroyo.arroyo_rs as native

class Document(SQLModel, table=True):
  path: str = Field(primary_key=True)
  file_hash: str

  headings: List[Heading] = Relationship(
    sa_relationship_kwargs={"cascade": "delete"},
    back_populates="document")
  keywords: List["Keyword"] = Relationship(
    sa_relationship_kwargs={"cascade": "delete"},
    back_populates="document")

  @classmethod
  def from_native_doc(cls, doc: native.Document):
    ret = cls(
      path=doc.path,
      file_hash=calculate_hash(doc.path),
    )

    return ret
# =arroyo.models.Document=:2 ends here

# [[file:../../arroyo-native-parser.org::*=arroyo.models.Document=][=arroyo.models.Document=:3]]
import hashlib

def calculate_hash(path: str) -> bytes:
  with open(path, "rb") as f:
    digest = hashlib.sha256(f.read())

  return digest.hexdigest()

def hash_updated(doc: Document):
  file_hash = calculate_hash(doc.path)
  return file_hash != doc.file_hash
# =arroyo.models.Document=:3 ends here
