# [[file:../../arroyo-native-parser.org::*=arroyo.models.Heading=][=arroyo.models.Heading=:1]]
from typing import Optional, List

from sqlmodel import Field, SQLModel, Relationship
from sqlmodel import Session

from arroyo.models.tag import Tag
from arroyo.models.reference import Reference

import arroyo.arroyo_rs as native

class Heading(SQLModel, table=True):
  id: Optional[int] = Field(default=None, primary_key=True)
  node_id: Optional[str]
  level: int
  text: str

  path: str = Field(foreign_key="document.path")
  document: Optional["Document"] = Relationship(
    sa_relationship_kwargs={"cascade": "delete"},
    back_populates="headings")

  tags: List[Tag] = Relationship(
    back_populates="heading",
    sa_relationship_kwargs={"cascade": "delete"})

  references: List[Reference] = Relationship(
    back_populates="heading",
    sa_relationship_kwargs={"cascade": "delete"})

  # refs
  # aliases
  # links

  @classmethod
  def from_parser(cls, path: str, src: native.Heading):
    return cls(
      node_id=src.id,
      level=src.level,
      text=src.text,

      path=path,
      # tags=make_tags_list(engine, src.tags or []),
    )

  @classmethod
  def from_native_doc(cls, doc: native.Document):
    return [
      cls.from_parser(doc.path, nhead)
      for nhead in doc.headings
    ]
# =arroyo.models.Heading=:1 ends here
