# [[file:../../arroyo-native-parser.org::*=arroyo.models.Keyword=][=arroyo.models.Keyword=:1]]
from typing import Optional, List

from sqlmodel import Field, SQLModel, Relationship
# =arroyo.models.Keyword=:1 ends here

# [[file:../../arroyo-native-parser.org::*=arroyo.models.Keyword=][=arroyo.models.Keyword=:2]]
import arroyo.arroyo_rs as native

class Keyword(SQLModel, table=True):
  path: str = Field(primary_key=True, foreign_key="document.path")
  keyword: str = Field(primary_key=True)
  value: str = Field(primary_key=True)

  document: Optional["Document"] = Relationship(
    sa_relationship_kwargs={"cascade": "delete"},
    back_populates="keywords")

  @classmethod
  def from_parser(cls, kw: native.Keyword):
    return cls(
      path=kw.file,
      keyword=kw.keyword,
      value=kw.value,
    )

  @classmethod
  def from_native_doc(cls, doc: native.Document):
    allowed_kws = [
      "AGE_OF_MONEY",
      "ARCOLOGY_ALLOW_CRAWL",
      "ARCOLOGY_FEED",
      "ARCOLOGY_KEY",
      "ARCOLOGY_TOOT_VISIBILITY",
      "ARROYO_DIRENV_DIR",
      "ARROYO_EMACS_MODULE",
      "ARROYO_EMACS_TANGLED",
      "ARROYO_FEEDS",
      "ARROYO_FEED_CATEGORY",
      "ARROYO_FEED_URL",
      "ARROYO_HOME_EPKGS",
      "ARROYO_HOME_MODULE",
      "ARROYO_MODULE_DEP",
      "ARROYO_MODULE_WANTED",
      "ARROYO_MODULE_WANTS",
      "ARROYO_NIXOS_EXCLUDE",
      "ARROYO_NIXOS_MODULE",
      "ARROYO_NIXOS_ROLE",
      "BIRTHDAY",
      "CCE_HOME_MODULE",
      "JPMORGAN",
      "LOCATION"
      "PRONOUNS",
      "SAVINGS",
      "SPENT_THIS_MONTH",
    ]

    def allowed_kw(kw: native.Keyword):
      return kw.keyword not in allowed_kws

    return [
      cls.from_parser(nkw)
      for nkw in doc.keywords
      if allowed_kw(nkw)
    ]
# =arroyo.models.Keyword=:2 ends here
